from django.db import models
from modelcluster.fields import ParentalKey
from wagtail.admin.panels import FieldPanel, InlinePanel, ObjectList, TabbedInterface
from wagtail.contrib.forms.models import AbstractForm, AbstractFormField
from wagtail.contrib.forms.panels import FormSubmissionsPanel
from wagtail.fields import StreamField
from wagtail.models import Page
from wagtailmetadata.models import MetadataPageMixin

from shared.models import (
    EcomailNewsletterMixin,
    ExtendedMetadataHomePageMixin,
    FbEventsMixin,
)
from shared.utils import subscribe_to_ecomail_newsletter
from shared_legacy.utils import make_promote_panels

from .blocks import CommonWishBlock, StatBlock, TourDateBlock


class MakeAWIshFormField(AbstractFormField):
    page = ParentalKey(
        "MakeAWishRoot", on_delete=models.CASCADE, related_name="form_fields"
    )


class MakeAWishRoot(
    AbstractForm,
    ExtendedMetadataHomePageMixin,
    MetadataPageMixin,
    EcomailNewsletterMixin,
    FbEventsMixin,
    Page,
):
    main_text = models.TextField(verbose_name="Hlavní text")
    success_text = models.TextField(verbose_name="Text po odeslání formuláře")

    header_image = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="+",
        verbose_name="Obrázek v headeru",
    )

    new_wish_image = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="+",
        verbose_name="Obrázek k novému přání",
    )

    video_url = models.URLField(
        verbose_name="URL Videa vpravo", help_text="Automaticky se spustí bez zvuku"
    )

    common_wishes = StreamField(
        [("wish", CommonWishBlock())], verbose_name="Nejčastější přání"
    )

    tour_dates = StreamField(
        [("tour_date", TourDateBlock())], verbose_name="Tour termíny"
    )

    stats = StreamField([("stat", StatBlock())], verbose_name="Statistiky")

    matomo_id = models.IntegerField(
        "Matomo ID pro sledování návštěvnosti", blank=True, null=True
    )

    content_panels = Page.content_panels + [
        FieldPanel("main_text"),
        FieldPanel("success_text"),
        FieldPanel("header_image"),
        FieldPanel("new_wish_image"),
        FieldPanel("video_url"),
        FieldPanel("common_wishes"),
        FieldPanel("tour_dates"),
        FieldPanel("stats"),
        InlinePanel("form_fields", label="Formulář - neměnit!!"),
    ]

    promote_panels = make_promote_panels() + [
        FieldPanel("matomo_id"),
        FieldPanel("ecomail_newsletter_list_id"),
        FieldPanel("ecomail_newsletter_list_tags"),
        FieldPanel("ecomail_newsletter_list_source"),
    ]

    advanced_panels = [FieldPanel("fbevents_id")]

    submissions_panels = [FormSubmissionsPanel()]

    edit_handler = TabbedInterface(
        [
            ObjectList(content_panels, heading="Obsah"),
            ObjectList(promote_panels, heading="Metadata"),
            ObjectList(submissions_panels, heading="Data z formuláře"),
            ObjectList(advanced_panels, heading="Pokročilé"),
        ]
    )

    @property
    def root_page(self):
        return self

    def process_form_submission(self, form):
        # Custom code on form submission
        cleaned_data = form.cleaned_data

        if cleaned_data["e_mail"] and self.ecomail_newsletter_list_id:
            name_and_surname = cleaned_data["jmeno_a_prijmeni"]
            name_and_surname = name_and_surname.split(" ")

            name = ""
            surname = ""

            if len(name_and_surname) > 1:
                name = name_and_surname[0]
                surname = " ".join(name_and_surname[1:])
            else:
                name = name_and_surname[0]

            custom_data = {
                "name": name,
            }

            if surname != "":
                custom_data["surname"] = surname

            subscribe_to_ecomail_newsletter(
                cleaned_data["e_mail"],
                self.ecomail_newsletter_list_tags,
                self.ecomail_newsletter_list_source,
                self.ecomail_newsletter_list_id,
                custom_data,
            )

        super().process_form_submission(form)

    class Meta:
        verbose_name = "Máte přání"
